/* SRE2001, ver 1.04 

This is the main program. It reads the initialization file, 
launches a "status" daemon, stores the "parent" and "child" procedures
to macrospace (for use as transient trsnaction & request-specific daemons), loads the 
main "filter" to macrospace, optionally calls a filter-specific startup procedure, and
optionally launches a filter specific daemon manager.

Then, it waits for requests, and launches transaction daemons to handle them.

*/

/***** User configurable parameters -- they usually do not need to be changed ***/

/* The SRE2001_CFG file contains several basic (not filter specific) parameters */
SRE2001_CFG='cfg\SRE2001.CFG'

/* if TZ variable not available, or not understood, use this
   GMT offset (in seconds). Note that this is the number of
   seconds that should be subtracted from local time to obtain GMT time.
   Example: -18000 for EST */
default_gmtoffset=-18000

/* Frequency (in seconds) to update status information */
update_rate=1

/* "target" time in seconds-- (if average response time>target_time, 
    or > 2* target_time, change color in display). */
target_time=1.5

/* percent of active clients at which warning color is used
   (on status screen)
   Should be a value between 0 and 1 */
client_warning=0.8

/* beep when a connection is opened */
beepon=0

/* how frequently to check that all daemons are functioning.
   Large numbers means less frequent checks (each value is about 5 second*/
daemon_check_freq=20

/*****  End of User configurable parameters ****/


/* 1) Load the various dlls, initializse, etc */

parse arg aopts; aopts=strip(aopts)

call load_dlls

val = SockInit()
if val \= 0 then do
   say "ERROR: SockInit failure: " errno
   exit
end
/* create a socket */
sock1 = SockSocket('AF_INET','SOCK_STREAM','IPPROTO_TCP')
if sock1 < 0 then do
   say 'ERROR: can not get socket (SockSocket:' errno
   exit
end

call sre_ini aopts

signal on syntax name abend ; signal on error name abend
signal on failure name abend ; signal on halt name doquit
signal on novalue name abend

default_gmtoffset0=default_gmtoffset


/* bind the socket to port */
addr.port =sreparams.!PORT 
addr.addr = 'INADDR_ANY'
addr.family = 'AF_INET'
prob=1
foo=socksetsockopt(sock1,'SOL_SOCKET','SO_REUSEADDR',prob)
val = SockBind(sock1,'addr.')
if val \= 0 then do
       say 'ERROR: Can not bind socket 'sock1':' errno
       say "Suggestion: use NETSTAT -s and look for a line of the form: "
say "      1268 STREAM         0    www-http..80         0.0.0.0  LISTEN"
       say "(this means that some other app is listening to port 80)"
       say "You should find the application and close it."
       say "Or (since this may be a left over from a failed app), just close the"
       say "the socket (say, using the SOCLOSE program)."
       val = SockShutdown(sock1,2)
       val = SockClose(sock1)
       exit 
end

/* make this socket 'passive' */
inq=max(sreparams.!LIMITCLIENTS,8)
val = SockListen(sock1,inq)
if val \= 0 then do
  say 'ERROR: can not listen on socket :' errno
  val = SockShutdown(sock1,2)
  val = SockClose(sock1)
  exit
end

if Sockioctl(sock1,'FIONBIO',1) = -1 then  do      /* switch to nonblocking mode */
   foo=sre_audit('SRE2001.CMD','crashed in sockioctl of sock1 'socket':'errno)
   signal errordone
end

/* almost ready to start accepting requsts. First invoke some daemons */
call do_daemons

/* set up the hostinfo procedure(s) */
foo=sre_hostinfo('!CLEAR')
IF abbrev(FOO,'error')=1 then DO
   SAY "ERROR initializing sre_hostinfo: "foo
   exit
end

/* filter specific initializations: procedure, and daemon manager */
call do_init_procs
if result='QUIT' then signal doquit

call syssleep(1.3)              /* wait a second */

schecked=0
transaction=0
show_wait=wordpos('SHOW_WAIT',sreparams.!status_items)

if wordpos('VERBOSE',sreparams.!status_items)>0 then do
   call pmprintf(" ------- ")
  foo=pmprintf('SRE2001 started at '||date('n')||' '||time('n'))
  call pmprintf("       using filter: "sreparams.!filter)
  if sreparams.!fast_filter<>0 then 
  call pmprintf("  and Fast filter: "sreparams.!fast_filter)
end 


foo=sre_eventvars('LASTRESET',dstamp,0)
foo=sre_eventvars('LASTIDLE',dstamp,0)

/* --- At Last -- Start accepting requests!   !!!!!!!   */

reqcs=sreparams.!req_cache_enable','sreparams.!req_cache_verify','sreparams.!req_cache_call_anyways','sreparams.!req_cache_record
lims=sreparams.!limitbody' 'sreparams.!limitheader' 'sreparams.!limittimewait' 'sreparams.!limittimeinactive' 'sreparams.!limittimetotal
miscparams=sreparams.!default_socket_timeout||','||sreparams.!tracking_on||','||SREPARAMS.!GMTOFFSET||','|| ,
           sreparams.!datadir||','||sreparams.!server_software

AA=SRE_FLAG_set('PAUSE',0,pid)


Do Forever              /**** Now enter infinite "accept requests" loop ***/

   /* wait for a connection on this socket */

   schecked=schecked+1             /* number of sockets processed since last check_status */
   is_update=SRE_FLAG_SET('RESET',,pid)       /* check reset flag */

/* Reset parameters? Check for reset flag, or every 200 connections */
   if is_update>0 |  schecked>200 then do     
      call check_status 1
      if result=1 then signal doquit
      schecked=0
   end 

   if show_wait>0 then foo=sre_screen_write('SHOW_WAIT')

  kct=1                 /* number of .01 second "polls" (no request outstanding */
  do forever                    /* non blocking socket -- so we have to poll it */
     sock2 = SockAccept(sock1)
     if sock2>0 then leave              /* got a request */

     kct=kct+1
     if (kct//500)=1 then do     /* check for flags every 5 inactive seconds */
         kct=2
         call check_status 
         if result=1 then signal doquit
         schecked=0
     end 

     call syssleep(0.01)                /* take a short break */
  end

   if sock2 < 0 then do
      say  "ERROR: Can not accept on socket :" errno
      val = SockShutdown(sock1,2)
      val = SockClose(sock1)
      signal doquit
   end

   if beepon then call charout,'07'x


/* check for # of active clients (actually, "transaction" daemons) */
  isactive=sre_clients()
  if isactive>=sreparams.!LIMITCLIENTS then do  /* too many clients */
     if wordpos('LIMITS',sreparams.!STATUS_ITEMS)>0 then do 
        foo=sre_write_error("SRE2001. Client limit exceeded at "transaction)
        foo=sre_countvars('LIMITS',1)
     end 
     rc=sockshutdown(sock2,2)
     rc = SockClose(sock2)
     iterate
  end

  caddrs=''   
  if check_bad=1 then do
      gotbad=0
      cc = SockGetPeerName(sock2,"caddr.!")
      is1=space(caddr.!addr,0)
      caddrs=is1||' '||caddr.!port  
      if badips.is1=1 then do
         gotbad=1
      end
      else do
        if badips.!wild>0 then do
           parse var is1 p1 '.' p2 '.' p3 '.' p4
           t1=p1||'.*'
           gotbad=badips.t1
           if gotbad<>1 & p2<>'' then do
                t1=p1||'.'||p2||'.*'
                gotbad=badips.t1
                if gotbad<>1 & p3<>'' then do
                   t1=p1||'.'||p2||'.'||p3||'.*'
                   gotbad=badips.t1
                end
            end
        end                     /* checking wilds */
      end                       /* exact match is bad */
      if gotbad=1 then do       /* this is a "bad" ip -- ignore the request */
         rc=sockshutdown(sock2,2)
         rc = SockClose(sock2)
         if verbose>0 then fp=sre_screen_write('MESSAGE','Denying connection from 'is1" at "time('n'))
         iterate
      end
   end

/* get numeric name of host */
   val = SockGetSockName(sock2,"saddr.!")
   if val \= 0 then do
      foo=sre_write_error("SRE2001. ERROR: cannot get host name :" errno)
      iterate
   end

/* get name of host */
   asaddr=saddr.!addr
   hname=hnames.asaddr
   if hname='' then do
      aa=sockgethostbyaddr(asaddr,"ahost.!")
      hname=ahost.!name
      if aa=0 then hname=asaddr
      hnames.asaddr=hname
   end

  transaction=transaction+1
  tlist=TRANSACTION'  'pid' 'sock2' 'asaddr' 'saddr.!port

  call launch_transaction_daemon

end             /* wait for next transaction */

call pmprintf(' SHOULD NEVER GET HERE IN SRE2001.cmd')

abend:
        say 'Syntax error, line' sigl':' rc
        foo=sre_write_error('SRE2001. Syntax error, line' sigl':' rc)
         exit

doquit:
  say normal
  say ' **** ,,, **** '
  foo=sre_flag_set('SHUTDOWN',1,pid)  /* just in case ... signal daemons that they should close */
  foo=sre_flag_set('#SHUTDOWN',1)     /* also used to signal daemons that they should close */


  atr=scrrows-8
  aa=scrclear(,,atr,1)
 say ' Leaving SRE2001 (on socket # 'sock1
 foo=sockshutdown(sock1,2)
 foo=sockclose(sock1)
 im=queued()
 do mm=1 to im
    pull .
 end 

call syssleep(2)        /* give daemons a chance to close by themselves */

/* kill all the threads */
/* how many threads are open... */
threadlist=RxListThreads()
nowtid=dostid()
nkilled=0
say "Killing residual threads ... "
do mm=1 to words(threadlist)
   atid=strip(word(threadlist,mm))
   if nowtid=atid then iterate
   wow=rxprckillthread(atid)
   if wow<>0 then
      say "Unable to kill thread: "atid ' ('wow
    else
        nkilled=nkilled+1
end 
say "   # of threads killed= "nkilled "( of "words(threadlist)-1")"


/* disconnect all daemons */
foo=sysini(sre2001_ini,'$DMNLIST','ALL:','SLIST')
didclose=0
wasclosed=0 ; noclose=0
do mm=1 to slist.0
   if sre_dmn_command(slist.mm,'CONNECTED')=0 then do
       wasclosed=wasclosed+1
       iterate
   end 
   foo=sre_dmn_command(slist.mm,'DISCONNECT')
   if foo=1 then  do
      didclose=didclose+1
   end
   else  do
      say 'Problem disconnecting 'slist.mm ' = ' foo
      noclose=noclose+1
end 
say 'Daemons: auto-closed= 'wasclosed ', forced close='didclose', couldn''t close= 'noclose

/* disconnect all flags */
foo=sysini(SRE2001_INI,'$FLAGLIST','ALL:','SLIST')
didclose=0
do mm=1 to slist.0
   foo=sre_flag_close(slist.mm,pid)
   if foo=1 then 
      didclose=didclose+1
   else 
      say 'Problem disconnecting flag 'slist.mm ' = ' foo
end 
say "# of flags closed= "didclose "( of "||slist.0||")"

/* remove environment variables */
nenv=dosenvlist('evars.')
do mm=1 to evars.0
   avar=strip(translate(evars.mm))
   if abbrev(avar,'SRE__')=0 then iterate
   foo=sre_value(avar,' ','SRE')
end



/* disconnect all lists */
foo=sysini(SRE2001_INI,'$LISTLIST','ALL:','SLIST')
didclose=0
do mm=1 to slist.0
gpp=sre_list_set(slist.mm,'CT')
   foo=sre_list_close(slist.mm,pid)
   if foo=1 then 
      didclose=didclose+1
   else 
      say 'Problem disconnecting list 'slist.mm ' = ' foo
end 
say "# of lists closed= "didclose "( of "||slist.0||")"


exit


/*********************************/
/*   launch a daemon to  handle this transaction 
    or, use an existing "dormant" transaction daemon */

launch_transaction_daemon:

try_agin:

dmn_stuff=sre_list_set('DORMANT_TRANS','POP',0,pid)
if dmn_stuff=0 then  signal new_daemon                /* no dormant daemons, so launch one */
if abbrev(dmn_stuff,'error')=1 then do
   call sre_write_error('SRE2001: recycling transaction error= 'oldq)
   signal new_daemon
end 

/* make sure this daemon is still alive */
if sre_dmn_command(dmn_stuff,'CONNECTED',0)=0 then signal try_agin /* get another from queue */

/* awaken dormant transation daemon, using this transaction info */
paramlist='WAKEUP'||asep|| ,
                 tlist||asep|| ,
                 caddrs||asep|| ,
                 hname||asep|| ,
                 miscparams||asep|| ,
                 lims||asep||  ,
                 sreparams.!status_items||asep|| ,
                 sreparams.!audit_items||asep|| ,
                 reqcs||asep|| ,
                 sreparams.!fast_filter

foo=sre_dmn_ask(dmn_stuff,paramlist,0)
if abbrev(foo,'error q')=1 then signal new_daemon /* wierd error, so launch a new daemon */
return 1

/* jump here to launch new daemon */
new_daemon:
signal on syntax name abend ; signal on error name abend
signal on failure name abend ; signal on halt name doquit
 
 this1='TRN_'||transaction
 afoo=sre_dmn_launch(this1,':SRE_MOD2',, ,
                     tlist,,
                     caddrs, ,
                     hname, ,
                     miscparams, ,
                     lims,  ,
                     sreparams.!status_items, ,
                     sreparams.!audit_items, ,
                     reqcs, ,
                     sreparams.!fast_filter)

 return 0


/*********************/
/* create some flags and lists */

do_flags:

AA=SRE_FLAG_open('MAIN',,pid)
if abbrev(aa,'error')>0 then do 
 say "ERROR: could not create main flag: " aa
 exit
end
foo=sysini(SRE2001_INI,'$FLAGLIST','MAIN',1)

AA=SRE_FLAG_open('RESET',,pid)
if abbrev(aa,'error')>0 then do 
 say "ERROR: could not create reset flag: " aa
 exit
end
foo=sysini(SRE2001_INI,'$FLAGLIST','RESET',1)


AA=SRE_FLAG_open('MODIFY',,pid)
if abbrev(aa,'error')>0 then do 
 say "ERROR: could not create modify flag: " aa
 exit
end
foo=sysini(sre2001_ini,'$FLAGLIST','MODIFY',1)

AA=SRE_FLAG_open('SHUTDOWN',,pid)
if abbrev(aa,'error')>0 then do 
 say "ERROR: could not create shutdown flag: " aa
 exit
end
foo=sysini(sre2001_ini,'$FLAGLIST','SHUTDOWN',1)


AA=SRE_FLAG_open('#SHUTDOWN')
if abbrev(aa,'error')>0 then do 
 say "ERROR: could not create shutdown local flag: " aa
 exit
end
foo=sysini(sre2001_ini,'$FLAGLIST','#SHUTDOWN',1)


AA=SRE_FLAG_open('PAUSE',,pid)
if abbrev(aa,'error')>0 then do 
 say "ERROR: could not create pause flag: " aa
 exit
end
foo=sysini(sre2001_ini,'$FLAGLIST','PAUSE',1)
AA=SRE_FLAG_set('PAUSE',1,pid)

AA=SRE_FLAG_open('#PAUSE_SYSVARS')
if abbrev(aa,'error')>0 then do 
  say "ERROR: could not create #pause_SYSVARS  local flag: " aa
  exit
end
foo=sysini(sre2001_ini,'$FLAGLIST','#PAUSE_SYSVARS',1)


AA=SRE_FLAG_open('#PAUSE_HOST')
if abbrev(aa,'error')>0 then do 
  say "ERROR: could not create #pause_Host  local flag: " aa
  exit
end
foo=sysini(sre2001_ini,'$FLAGLIST','#PAUSE_HOST',1)


ss=sre_list_open('DORMANT_TRANS',pid)
if abbrev(ss,'error')>0 then do 
  say "ERROR: could not create DORMANT_TRANS list: " aa
  exit
end
foo=sysini(sre2001_ini,'$LISTLIST','DORMANT_TRANS',1)

ss=sre_list_open('ACTIVE_TRANS',pid)
if abbrev(ss,'error')>0 then do 
  say "ERROR: could not create ACTIVE_TRANS list: " aa
  exit
end
foo=sysini(sre2001_ini,'$LISTLIST','ACTIVE_TRANS',1)

ss=sre_list_open('DORMANT_REQ',pid)
if abbrev(ss,'error')>0 then do 
  say "ERROR: could not create DORMANT_REQ list: " aa
  exit
end
foo=sysini(sre2001_ini,'$LISTLIST','DORMANT_REQ',1)


return 1



/*********************/
/* setup daemon stuff */

do_daemons:

own_id=sre_dmn_own(2)   /* used by main thread when talking to daemons */
if abbrev(own_id,'error')=1 then   do
    foo=sre_write_error('SRE2001.  WARNING: own id error '||own_id)
    exit
end
parse var own_id isnew','own_id
if isnew=1 then foo=sysini(sre2001_ini,'$DMNLIST',own_id)


/* launch process-wide variable storage daemon */
var_dmnid=sre_dmn_launch('VAR',bin_dir||'\SRE_VAR.RXX',10000,dstamp,tempdir)
  if abbrev(var_dmnid,'error')=1 then do
     say  ' Problem with Variable-storage Daemon:  ' var_dmnid
     exit
  end
  else do
     parse var var_dmnid atid ',' s1 ',' verify
     if verify<>1 then do
          say  ' Problem verifying VAR  Daemon:  ' verify
         exit
     end 
     say "Variable-storage daemon started on thread "atid
    foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
   end 

  dmnlist.!var=var_dmnid

/*  launch audit daemon */
aud_dmnid=sre_dmn_launch('AUDIT',bin_dir'\SRE_AUDI.RXX',10000, ,
                        sreparams.!tracking_on,sreparams.!auditfile,,
                        sreparams.!req_cache_hitfile,sreparams.!req_cache_record,sreparams.!audit_delay)
  if abbrev(aud_dmnid,'error')=1 then do
     say  ' Problem with Audit Daemon:  ' aud_dmnid
     exit
  end
  else do
     parse var aud_dmnid atid ',' s1 ',' verify
     if verify<>1 then do
          say  ' Problem verifying Audit Daemon:  ' verify
         exit
     end 
     say "Audit daemon started on thread "atid
    foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
   end 

   dmnlist.!aud=aud_dmnid

/* Launch caching daemons 
request: the request cache (match selector to a file)
ipname: match ip address to ip name
count: counter variables storage (such as BYTESENTTOTAL)
event: event variables storage (such as LASTACCEPT)
req_info: daemon info for request daemons
tran_info: daemon info for transaction daemons
*/

csh_dmnid=sre_dmn_launch('CACHE',bin_dir'\SRE_CSH.RXX',10000, ,
                         'REQ REQUEST,IPNAME,COUNT COUNTS,EVENT EVENTS, REQ_INFO, TRAN_INFO ')
  if abbrev(csh_dmnid,'error')=1 then do
    say  ' Problem with Cache  Daemon:  ' csh_dmnid
    exit
  end
  else do
     parse var csh_dmnid atid ',' s1 ',' verify
     if verify<>1 then do
          say  ' Problem verifying Cache  Daemon:  ' verify
          exit
     end 
     say "Cache daemon started on thread "atid
     foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
  end 

  dmnlist.!csh=csh_dmnid

/* set sizes of various caches */
 foo=sre_cache('REQUEST','RESET',sreparams.!req_cache_size,,,own_id)
 foo=sre_cache('IPNAME','RESET',sreparams.!maxclientnames,,,own_id)
 foo=sre_cache('COUNT','RESET',sreparams.!maxcounters,,,own_id)
 foo=sre_cache('EVENT','RESET',sreparams.!maxcounters,,,own_id)


if sreparams.!MaxMD5s>0 then do
  md5_dmnid=sre_dmn_launch('SRE_MD5',bin_dir'\SRE_MD5.RXX',10000,sreparams.!maxmd5s)
  if abbrev(md5_dmnid,'error')=1 then do
    say  ' Problem with MD5 Daemon:  ' md5_dmnid
    exit
  end
  else do
     parse var md5_dmnid atid ',' s1 ',' verify
     if verify<>1 then do
          say  ' Problem verifying MD5  Daemon:  ' verify
          exit
     end 
     say "MD5 daemon started on thread "atid
     foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
  end 
end 

   dmnlist.!md5=md5_dmnid

/* Launch queue daemon (which is used for statvals) */
que_dmnid=sre_dmn_launch('QUEUE',bin_dir'\SRE_QUE.RXX',10000)
  if abbrev(que_dmnid,'error')=1 then do
    say  ' Problem with Que daemon:  ' que_dmnid
    exit
  end
  else do
     parse var que_dmnid atid ',' s1 ',' verify
     if verify<>1 then do
          say  ' Problem verifying Queue Daemon:  ' verify
          exit
     end 
     say  "Queue daemon started on thread "atid
     foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
  end 
  
  dmnlist.!que=que_dmnid

/* set sizes of various queues 
  bytes: bytes sent in this request
  total_time: time neede to resolve this request (from launch of daemon to closing out request
  start_transmit_time: "setup" time (from launch of daemon till first response packet sent 
*/
 foo=sre_queue('BYTES','CREATE',sreparams.!maxstatvals ,0)
 foo=sre_queue('TOTAL_TIME','CREATE',sreparams.!maxstatvals,0)
 foo=sre_queue('START_TRANSMIT_TIME','CREATE',sreparams.!maxstatvals,0)
 foo=sre_queue('REQUEST','CREATE',sreparams.!maxstatvals,0)
 foo=sre_queue('CLIENTADDR','CREATE',sreparams.!maxstatvals,0)


/* Launch status daemon */
  asep='01'x
  srep=sreparams.!port||asep||sreparams.!no_modify_onthefly
  srep=srep||asep||sreparams.!status_message||asep||sreparams.!status_items
  srep=srep||asep||sreparams.!server_software||asep||sreparams.!req_cache_enable
  srep=srep||asep||sreparams.!tracking_on

  if datatype(client_warning)<>'NUM' then client_warning=1
  if client_warning<0 | client_warning>1 then client_warning=1
  client_warning=client_warning*sreparams.!limitclients

  stat_dmnid=sre_dmn_launch('STATUS',bin_dir'\SRE_STAT.RXX',0,,
             sreparams.!filter' 'sreparams.!fast_filter, ,
             sock1,update_rate, ,
             target_time,client_warning,SRE2001_CFG,,
             srep)
  if abbrev(stat_dmnid,'error')=1 then do
     say  ' Problem with Status Daemon:  ' stat_dmnid
  end
  else do
     parse var stat_dmnid atid ',' s1 ',' .
     say "Status daemon started on thread "atid
     foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
  end 

  dmnlist.!stat=stat_dmnid

  dmnlist.0='!AUD !VAR !CSH !MD5 !QUE !STAT' 


return 1

/********************************/
/* status check */
check_status:
parse arg is_update

/* check for "pause */
npp=0
do forever
   if sre_flag_set('PAUSE',,pid)=1 then do
       if npp=0 then  foo=sre_screen_write('MESSAGE','Paused at '||time('n'))
       foo=syssleep(0.1)
       npp=npp+1
       iterate
   end 
   leave
end

if npp>0 then  call  foo=sre_screen_write('MESSAGE','Ready to accept new connections')

if sre_flag_set('SHUTDOWN',,pid)=1 then do
     aa=sre_flag_set('#SHUTDOWN',1)
     rc=sockshutdown(sock1,2)
     rc = SockClose(sock1)
      foo=sre_screen_write('MESSAGE','SHUTDOWN order has been received ')
     do mm=1 to 45              /* give it 90 seconds to complete */
           if sre_clients()=0 then leave  /* no more clients */
           call syssleep(2)
           call  foo=sre_screen_write('MESSAGE','Waiting for clients to finish: '(mm*2) seconds  ' of 90 ')
     end
     return 1
end 


/* check for "reset" signal */
if is_update='' then
   is_update=sre_flag_set('RESET',,pid)


if is_update>0 then do
   aa=sre_flag_set('RESET',0,pid)
   call update_sre_ini

   if result=0 then return 1            /* some kind of error */
   if pos('RESET',sreparams.!audit_items)>0 then foo=sre_audit('RESET ',time('n')||' '||date('n'),0,1)

   Foo=date('S')
   adate=left(foo,4)||'.'||substr(foo,5,2)||'.'||substr(foo,7,2)
   asec=time('N')
   dstamp=adate||' '||asec
   foo=sre_eventvars('LASTRESET',dstamp,0)


   reqcs=sreparams.!req_cache_enable','sreparams.!req_cache_verify','sreparams.!req_cache_call_anyways','sreparams.!req_cache_record
   lims=sreparams.!limitbody' 'sreparams.!limitheader' 'sreparams.!limittimewait' 'sreparams.!limittimeinactive' 'sreparams.!limittimetotal
   miscparams=sreparams.!default_socket_timeout||','||sreparams.!tracking_on||','||SREPARAMS.!GMTOFFSET||','|| ,
           sreparams.!datadir||','||sreparams.!server_software
end 

if sreparams.!Unallowed_file<>0 then do
  aa=stream(sreparams.!unallowed_file,'c','query datetime')
  if aa<>badips.!datetime then do
     call read_unalloweds
     if verbose>0 then  foo=sre_screen_write('MESSAGE','# of unallowed IP entries ='||badips.!total)
  end 
end 

/* check for daemon life? */
daemon_check=daemon_check+1
if daemon_check>daemon_check_freq then do

   if daemon_check_warning<>0  then do  /* double check suspicious daemon */
        admnid=dmnlist.daemon_check_warning
        astat=sre_dmn_command(admnid,'VERIFY',4000)
        if astat<>1 then do
             foo=sre_write_error('SRE2001. ERROR: non responding daemon 'aw,2) 
             call pmprintf('ERROR: non responding daemon 'aw) 
             return 1   /* second error -- quit! */
         end
   end 

/* even if suspicious daemon is okay, check all of them again */
    do ww=1 to  words(dmnlist.0)
        aw=strip(word(dmnlist.0,ww))
        admnid=dmnlist.aw
        astat=sre_dmn_command(admnid,'VERIFY',2000)
        if astat<>1 then do
            foo=sre_write_error('SRE2001. Warning: non responding daemon 'aw,2) 
            call pmprintf('Warning: non responding daemon 'aw) 
            daemon_check_warning=aw
            daemon_check=Max(daemon_check_freq-2,1)  /* check again soon */
            return 0
        end
    end 

/* if here, daemons are okay */
    daemon_check=0    ;daemon_check_warning=0                  /* reset the clock ... */

end                     /* daemon_check > check_freq */

return 0

/******************************/
/* initialize, load various dlls, etc. */
sre_ini:

crlf='0d0a'x
tsreparams.=''
dmn_tid=0               /* signals no daemon manager */
blank = ' '
null = ''
asep='01'x
daemon_check=0
daemon_check_warning=0

parse value scrsize() with scrrows scrcols

basicqueue=rxqueue('G')

/* currently required variables in SRE2001_CFG file */
srevars='PORT DATADIR LIMITBODY LIMITCLIENTS LIMITHEADER LIMITTIMETOTAL ' ,
        'LIMITTIMEWAIT LIMITTIMEINACTIVE FILTER FAST_FILTER DAEMON_MANAGER INIT_PROC ' ,
        'DEFAULT_SOCKET_TIMEOUT  SERVER_SOFTWARE STATUS_ITEMS ERRORFILE AUDITFILE AUDIT_ITEMS ',
        'SERVER_SOFTWARE_SHORT UNALLOWED_FILE NO_MODIFY_ONTHEFLY AUDIT_DELAY ' ,
        ' MAXMD5S MAXCOUNTERS MAXCLIENTNAMES MAXSTATVALS', 
        ' REQ_CACHE_SIZE REQ_CACHE_CALL_ANYWAYS REQ_CACHE_ENABLE ',
        ' REQ_CACHE_VERIFY REQ_CACHE_RECORD REQ_CACHE_HITFILE '  ,
        ' STATUS_MESSAGE TRACKING_ON  '


/* enable ansi screen controls */
aesc='1B'x
cy_ye=aesc||'[37;46;m'
cyanon=cy_ye
normal=aesc||'[0;m'
bold=aesc||'[1;m'
re_wh=aesc||'[31;47;m'
reverse=aesc||'[7;m'
call charout, '1b5b'x||'2J'

if aopts='?' then do
    call showopts
    exit
end 

say "            "cy_ye"SRE2001 ver 1.04 "normal
say "    To view options. enter "bold"x:>SRE2001 ? "normal

say
say bold" Initializing ... "normal


pid=dospid()

taopts=translate(aopts)

/* alternative SRE2001_CFG file? */
if pos('-I ',taopts)>0 then do
   parse var taopts  '-I' SRE2001_CFG
   SRE2001_CFG=strip(SRE2001_CFG)
   say "   Using initialization file: " SRE2001_CFG
end 

if pos('-P ',taopts)>0 then do
   parse var taopts  '-P' apriority
   parse var apriority lvl '_' clas ; lvl=strip(lvl) ; clas=strip(clas)
   if datatype(clas)<>'NUM' then do
        say "A Bad Priority Class (must be between 1 and 4): "clas
        exit
   end 
   if datatype(lvl)<>'NUM' then do
        say "A Bad Priority Level (must be between 0 and 31): "lvl
        exit
   end 
   if clas>4 | clas<1  then do
        say "Bad Priority Class (must be between 1 and 4): "clas
        exit
   end 
   if lvl>31 | lvl<0  then do
        say "Bad Priority Level (must be between 0 and 31): "lvl
        exit
   end 
   apid=dospid()
   foo=translate(strip(dospriority(lvl,clas,apid)),'/',' ')
   foo2=translate(strip(dospriority(,0,apid)),'/',' ')
   say "   Changed priority level/class from "foo " to " foo2
end 

If pos('-CVT',taopts)>0 then do   /* convert from goseve to SRE2001 */
  call charout, '1b5b'x||'2J'
  oo=gos2sre(' ')
  say
  say " Conversion(s) completed. "
  exit
end


/* modify initialization file (call MOD_INI.CMD) */
if pos('-M',taopts)>0 then do
  if pos('-MALL',taopts)=0 then do
     parse var taopts . '-M' daopt .
    if daopt=''  then daopt=SRE2001_CFG
    foo="bin\mod_ini"(daopt' 2')
    say
    say " Modifications completed. Status: "bold||foo||normal
    exit
  end
  parse var taopts . '-MALL' daopt .
  if daopt=''  then daopt=SRE2001_CFG
  foo="bin\mod_ini"(daopt,asrevars)
  say
  say " Modifications completed. Status: "bold||foo||normal
  exit
end

/* read initialization parameters */

call read_cfg_file

/* clear ini file */
appname='$SYSVARS'
sininame='temp\sre'||sreparams.!port||'.ini'
foo=sysini(sininame,appname,'DELETE:')   /* initialize 'SRE2001.INI */
if foo='ERROR:' then do
   say "Unable to clear 'appname'in 'SRE2001 INI file "
   exit
end 
sre2001_ini=stream(sininame,'c','query exists')
if sre2001_ini='' then do
   say "Error: problem creating sre2001_ini "
   exit
end 
foo=tsre_sysvars('SRE2001_INI',sre2001_ini)

appname='$DMNLIST'
sininame='temp\sre'||sreparams.!port||'.ini'
foo=sysini(sininame,appname,'DELETE:')   /* initialize 'SRE2001.INI */
if foo='ERROR:' then do
   say "Unable to clear 'appname'in 'SRE2001 INI file "
   exit
end 
appname='$FLAGLIST'
foo=sysini(sininame,appname,'DELETE:')   /* initialize 'SRE2001.INI */
if foo='ERROR:' then do
   say "Unable to clear 'appname'in "sininame " file "
   exit
end 

foo=tsre_sysvars('LASTSTART',dstamp)
foo=tsre_sysvars('LISTEN_SOCK',sock1)


/* create some flags */
call do_flags


/* initialize  some static storage  */

/* temp storage, for use in creating macrospace proc 
   These are also stored to SRE2001.ini */
do mm=1 to words(srevars)
   aw=strip(word(srevars,mm))
   aw2='!'||aw
   foo=tsre_sysvars(aw,sreparams.aw2)
end 


/* determine gmtoffset using TZ? */
tz=value('TZ',,'os2environment')
if tz='' then do
      say 'ERROR TZ variable is not available'
          say "    Using default GMTOFFSET= " default_gmtoffset
      say bold"    You should add TZ to CONFIG.SYS, or SET TZ before running SRE2001"normal
      sreparams.!gmtoffset=default_gmtoffset
end
else do
   sreparams.!gmtoffset=fig_gmtoffset(tz,default_gmtoffset0)
   say "  Using GMT Offset = "sreparams.!gmtoffset
   default_gmtoffset0=sreparams.!gmtoffset
end

oo=sreparams.!GMTOFFSET
foo=tsre_sysvars('GMTOFFSET',oo)  


/* set some directory stuff */
/* SERVDRIVE SERVDIR TEMPDIR BIN_DIR CFG_DIR LOG_DIR */

maindir=directory()
maindir=translate(maindir,'\','/')
maindir=strip(maindir,,'\')
foo=tsre_sysvars('SERVDIR',maindir)
foo=tsre_sysvars('SRV_DIR',maindir)

maindrive=filespec('D',maindir)
foo=tsre_sysvars('SERVDRIVE',maindrive)

/* check for existence of required subdirectories */

tempdir=maindir||'\TEMP'
if dosisdir(tempdir)=0 then do
    say "ERROR: no \TEMP subdirectory ("tempdir')'
    exit
end 
foo=tsre_sysvars('TEMPDIR',tempdir)
foo=tsre_sysvars('TMP_DIR',tempdir)
foo=tsre_sysvars('TEMP_DIR',tempdir)

bin_dir=maindir||'\BIN'
if dosisdir(bin_dir)=0 then do
    say "ERROR: no \BIN subdirectory ("bin_dir')'
    exit
end 
foo=tsre_sysvars('BIN_DIR',bin_dir)

cfg_dir=maindir||'\CFG'
if dosisdir(cfg_dir)=0 then do
    say "ERROR: no \CFG subdirectory ("cfg_dir')'
    exit
end 
foo=tsre_sysvars('CFG_DIR',cfg_dir)

log_dir=maindir||'\LOG'
if dosisdir(log_dir)=0 then do
    say "ERROR: no \LOG subdirectory ("log_dir')'
    exit
end 
foo=tsre_sysvars('LOG_DIR',log_dir)


/*  cache hit file */

if pos(':',sreparams.!req_cache_hitfile)=0 then do
   if abbrev(sreparams.!req_cache_Hitfile,'\')=0 then do
       sreparams.!req_cache_hitfile=maindir'\'sreparams.!req_cache_hitfile
       arf=stream(sreparams.!req_cache_hitfile,'c','query exists')
       if arf='' then do
            aa=';SRE2001 cache hit file '
            foo=charout(sreparams.!req_cache_Hitfile,aa,1)
            foo2=stream(sreparams.!req_cache_hitfile,'c','close')
            if foo=0 then do
                 say 'Cache-hit file created: 'sreparams.!req_cache_hitfile
            end 
            else do
                 say 'ERROR: unable to use, or create, cache-hit file: 'sreparams.!req_cache_hitfile
                exit
            end 
       end 
       foo=tsre_sysvars('REQ_CACHE_HITFILE',sreparams.!req_cache_hitfile)  
       say '  Using hit-cache file: 'sreparams.!req_cache_hitfile
   end
end

/* check existence of datadir (root of web tree) */
datadir=sreparams.!DATADIR
datadir=translate(datadir,'\','/')
datadir=strip(strip(datadir,'t','\'))
if pos(':',datadir)=0 then do
   if abbrev(datadir,'\')=1 then do
        datadir=maindrive||datadir
   end
   else do
      datadir=maindir||'\'||datadir
   end /* do */
end
dd=dosisdir(datadir)
if dd=0 then do
   say 'ERROR: no such datadir ('datadir
   exit
end
sreparams.!datadir=datadir

foo=tsre_sysvars('DATADIR',datadir)  


/* check for existence of filter */
filter=stream(maindir||'\'||sreparams.!FILTER,'c','query exists')
if filter='' then do
   say 'ERROR: no such filter file ('filter
   exit
end
sreparams.!FILTER_DIR=strip(filespec('D',filter)||filespec('P',filter),,'\')
foo=tsre_sysvars('FILTER_DIR',sreparams.!FILTER_DIR)  

/* check for existence of FAST filter */
fast_filter=sreparams.!FAST_FILTER
if FAST_FILTER<>0 then do
  fast_filter=stream(maindir||'\'||FAST_FILTER,'c','query exists')
   if fast_filter='' then do
     say 'ERROR: no such fast filter file ('fast_filter
     exit
   end
end
if fast_filter<>0 then do
  foo=macrodrop('SRE_FAST_FILTER')
  foo=macroadd('SRE_FAST_FILTER',fast_filter,'B')
  if foo<>1 then do
     say 'ERROR: unable to load fast filter ('fast_filter
     exit
  end
end

say "  Using filter: "filter
if fast_filter<>0 then
   say '    and fast filter:'fast_filter||normal

say "  Using data directory: "datadir

/* set some counters to 0 */
alist='BYTESREADTOTAL  BYTESSENTTOTAL TRANSACTIONS  REQUESTS LIMITS ERRORS BROKEN CACHEHITS ' 
do mm=1 to words(alist)
   aww=strip(word(alist,mm))
   foo=sre_countvars(aww,0)
   IF abbrev(FOO,'error')=1 then do
      say 'ERROR initializing counter variable 'aww '='foo
      exit
   end 
end 

/* load sre_filter */
foo=macrodrop('SRE_FILTER')
foo=macroadd('SRE_FILTER',filter,'b')
if foo=0 then do
 say "ERROR could not load filter into macrospace "
 exit
end


/* cache "request daemon procedure" in mspace */
foo=macrodrop("SRE_MOD3")
foo=macroadd("SRE_MOD3",bin_dir'\SRE_REQ.RXX','B')
if foo=0 then do
  say "ERROR could not load "bin_dir"\SRE_REQ.RXX into macrospace "
  exit
end

/* cache "transaction daemon procedure" in mspace */
foo=macrodrop("SRE_MOD2")
foo=macroadd("SRE_MOD2",bin_dir'\SRE_TRAN.RXX','B')
if foo=0 then do
  say "ERROR could not load ' bin_dir'\SRE_TRAN.RXX into macrospace "
  exit
end

hnames.=''

/* Read unallowed ips */
badips.=''
badips.!wild=0
check_bad=0
if sreparams.!unallowed_File<>0 then do
   call read_unalloweds
   check_bad=badips.!do
   say '  # of unallowed IP entries: 'badips.!total '( 'badips.!wild ' wildcards)'
end 


/* set default servername and serveraddress */
serveraddr=sockgethostid()
rc=sockgethostbyaddr(serveraddr,'foo.!')
if rc=0 then
  serveraname=foo.!name
else
  servername=serveraddr

sz=stream(sreparams.!auditfile,'c','query size')
if sz>0 then do
  aa=charin(sreparams.!auditfile,sz,1)
  if aa='1a'x then sz=sz-1
end
else do
   sz=1
end 
foo=charout(sreparams.!auditfile,'0D0A'X||'0d0a'x||'**** SRE2001 started at '||time('n')||' '||date('n')||'  ',sz)  
foo=stream(sreparams.!auditfile,'c','close')
if stream(sreparams.!auditfile,'c','query exists')='' then do
    say "ERROR: Unable to create audit file: " sreparams.!auditfile
    exit
end

sz=stream(sreparams.!errorfile,'c','query size')
if sz>0 then do
  aa=charin(sreparams.!errorfile,sz,1)
  if aa='1a'x then sz=sz-1
end
else do
   sz=1
end 
foo=charout(sreparams.!errorfile,'0D0A'X||'0d0a'x||'**** SRE2001 started at '||time('n')||' '||date('n')||'  '||'0d0a'x,sz)  
foo=stream(sreparams.!errorfile,'c','close')
if stream(sreparams.!errorfile,'c','query exists')='' then do
    say "ERROR: Unable to create error file: " sreparams.!errorfile
    exit
end

/* save "tsreparams" parameters "into macrospace" */
foo=save_tsreparams(tempdir)
if foo=0 then exit

/* now save a few things to the enviornment */
a=sre_value('PORT',sreparams.!port,'SRE')
a=sre_value('TRACKING',sreparams.!tracking_on,'SRE')


say bold 'Initialization completed for 'normal||'"'||sreparams.!server_software||'"'
call syssleep(1.5)
return 0

/*************/
/* run some initialization procedures */
do_init_procs:

if sreparams.!init_proc<>0 & sreparams.!init_proc<>'' then do
  aproc=maindir||'\'||strip(sreparams.!init_proc,,'\')
  foo=sre_screen_write('MESSAGE','  Calling custom initialization procedure  ('aproc') ....')
  foo=macrodrop('SRE2_INITPROC')
  foo=macroadd('SRE2_INITPROC',aproc,'b')
  if foo=0 then do
     foo=sre_write_error("SRE2001. Error: unable to load initialization procedure "aproc)
     call pmprintf ("Error: unable to load "aproc)
     foo=macrodrop('SRE2_INITPROC')
     return 'QUIT'
  end 
  foo=sre2_initproc(sreparams.!port)
  if foo=0  then return 'QUIT'
  foo=sre_screen_write('MESSAGE',"  Status from initialization procedure: "bold||foo||normal)
  foo=macrodrop('SRE2_INITPROC')
end


/* start daemon manager? */

if sreparams.!daemon_manager<>0 & sreparams.!daemon_manager<>'' then do
  afilt2=maindir'\'||strip(sreparams.!daemon_manager,,'\')
 foo=sre_screen_write('MESSAGE','  Starting SRE daemon manager ('afilt2') ....')

 aa=sre_flag_set('MAIN',0,pid)

 dman_id=sre_dmn_launch('DMN_MANAGER',afilt2,,'MAIN','SHUTDOWN',pid,maindir,sreparams.!PORT,servername)
  if abbrev(dman_id,'error')=1 then do
     say  ' Problem with  Daemon Manager:  ' dman_dmnid
     exit
  end
  else do
     parse var dman_id atid ',' s1 ',' .
     foo=sre_screen_write('MESSAGE',"Daemon manager daemon started on thread "atid)
     foo=sysini(sre2001_ini,'$DMNLIST',s1,1)
  end 

/* wait for daemon manager to do it's business */
  iarf=0
   do forever                           /* wait for daemon monitor to say "go ahead" */
      iarf=iarf+1 
      foo=sre_flag_set('MAIN',,pid)
      if foo=1 then leave
      foo=syssleep(1)
      if iarf//10=1 then
           foo=sre_screen_write('MESSAGE','  Waiting on initializations by daemon manager ('||IARF||'s)...')
   end    
end

if sre_flag_set('SHUTDOWN')=1 then do
   say "Daemon manager signalled a shutdown: "aa
   return 'QUIT'
end
 foo=sre_screen_write('MESSAGE','  Ready to accept new connections ')

return 1

/*********************/
/* read initialization parameters */
read_cfg_file:

foo=strip(translate(stream(SRE2001_CFG,'c','open read')))
if abbrev(foo,'READY')=0 then do
   say reverse'ERROR:'normal' can not open initialization file: 'SRE2001_CFG
   exit
end 
isz=stream(SRE2001_CFG,'c','query size')
if (isz=0 | isz='') then do
   say reverse'ERROR:'normal' empty or missing initialization file: 'SRE2001_CFG
   exit
end 

stuff=charin(SRE2001_CFG,1,isz)
foo=stream(SRE2001_CFG,'c','close')

sreparams.=''

/* now parse the file, eventually to save to variable storage space */

do until stuff=''
   parse var stuff aline (crlf) stuff
   if aline='' then iterate
   aline=strip(translate(aline,' ','0d0a09'x))
   if abbrev(aline,';')=1 then iterate
   parse var aline avar '=' aval ; 
   avar=translate(strip(avar)) ; aval=strip(aval)
   if wordpos(avar,srevars)=0 then do
      say "  "reverse"Warning:"normal"Unknown SRE2001_CFG variable: "aline
      exit
   end
   cname='!'||avar
   sreparams.cname=aval
end 

/* quality checks */

sreparams.!status_items=translate(space(sreparams.!status_items,1))
sreparams.!audit_items=translate(space(sreparams.!audit_items,1))

if datatype(sreparams.!default_socket_timeout)<>'NUM' then do
   say 'ERROR Bad socket timeout value : 'sreparams.!default_socket_timeout
   exit
end 


nerrs=0
do mm=1 to words(srevars)
   aw=strip(word(srevars,mm))
   aw2='!'||aw
   if sreparams.aw2='' then do
      say "ERROR: Unspecified Configuration variable: "aw
      say "Please edit "sre2001_cfg
      say "(or run SRE2001 -mall if you want to use a simple parameter editor)"
      exit
   end
end /* do */
if nerrs>0 then exit            /* missing ini variables */


/* check a few parameters */

if datatype(sreparams.!req_cache_size)<>'NUM' then sreparams.!req_cache_size=0

if sreparams.!req_cache_size=0 | datatype(sreparams.!req_cache_enable)<>'NUM' then
        sreparams.!req_cache_enable=0
if datatype(sreparams.!req_cache_record)<>'NUM' then sreparams.!req_cache_record=0


/* check limitbody and limitheader */
if datatype(sreparams.!limitbody)<>'NUM' then do
   say 'ERROR: LIMITBODY must be an integer > 0 '
   exit
end 
sreparams.!limitbody=sreparams.!limitbody*1000

/* check limitbody and limitheader */
if datatype(sreparams.!limitheader)<>'NUM' then do
   say 'ERROR: LIMITHEADER must be an integer > 0 '
   exit
end 
sreparams.!limitheader=sreparams.!limitheader*1000


/* set startup date */
Foo=date('S')
adate=left(foo,4)||'.'||substr(foo,5,2)||'.'||substr(foo,7,2)
asec=time('N')
dstamp=adate||' '||asec


return 1


/****************/
/* read unallowed_file */
read_unalloweds:

badips.=''
badips.!wild=0
badips.!do=1
badips.!total=0
ifoo=stream(sreparams.!unallowed_file,'c','query size')
if ifoo='' | ifoo=0 then do
    say "ERROR: could not find UnAllowed IP addresses file:  "sreparams.!unallowed_File 
   exit
end 
badips.!datetime=stream(sreparams.!unallowed_file,'c','query datetime')
sfoo=charin(sreparams.!unallowed_file,1,ifoo)
foo=stream(sreparams.!unallowed_file,'c','close')
do until sfoo=''
   parse var sfoo anip '0d0a'x sfoo ; 
   if anip='' then iterate
   anip=strip(word(anip,1))
   if abbrev(anip,';')=1 then iterate
   if pos('*',anip)>0 then do
       parse var anip a1 '*' .
       a1=strip(strip(a1),,'.')||'.*'
       badips.a1=1
       badips.!wild=badips.!wild+1
       badips.!total=badips.!total+1
   end 
   else do
       badips.anip=1
       badips.!total=badips.!total+1
   end
end

return 0


/**************************/
/* update SRE2001_CFG vars */
update_sre_ini:procedure expose sreparams. SRE2001_CFG maindir maindrive sock1  pid tz , 
                 default_gmtoffset0 basicqueue filter fast_filter tsreparams. tempdir sre2001_ini

crlf='0d0a'x

/* wait until no requests are active, or 1 minute */
iok=0

srevars_resetable='DATADIR LIMITBODY LIMITCLIENTS LIMITHEADER LIMITTIMETOTAL ' ,
        'LIMITTIMEWAIT LIMITTIMEINACTIVE ' ,
        'SERVER_SOFTWARE SERVER_SOFTWARE_SHORT REQ_CACHE_ENABLE REQ_CACHE_RECORD ' 

/* read initialization parameters */

foo=strip(translate(stream(SRE2001_CFG,'c','open read')))
if abbrev(foo,'READY')=0 then do
   foo=sre_write_error('SRE2001. INI error: Can not open initialization file')
   return 1
end 
isz=stream(SRE2001_CFG,'c','query size')
if (isz=0 | isz='') then do
   foo=sre_write_error('SRE2001. INI error: Empty or missing initialization file')
   return 1
end 

stuff=charin(SRE2001_CFG,1,isz)
foo=stream(SRE2001_CFG,'c','close')

/* now parse the file, eventually to save to SRE_SYSVARS cache */

do until stuff=''
   parse var stuff aline (crlf) stuff
   if aline='' then iterate
   aline=strip(translate(aline,' ','0d0a09'x))
   if abbrev(aline,';')=1 then iterate
   parse var aline avar '=' aval ; 
   avar=translate(strip(avar)) ; aval=strip(aval)
   if wordpos(avar,srevars_resetable)=0 then iterate  /* only change "resetable" variables */
   cname='!'||avar
   sreparams.cname=aval
end 

do mm=1 to words(srevars_resetable)
   aw=strip(word(srevars_resetable,mm))
   aw2='!'||aw
   foo=tsre_sysvars(aw,sreparams.aw2)
end 

/* determine gmtoffset using TZ? */
sreparams.!gmtoffset=fig_gmtoffset(tz,default_gmtoffset0)

foo=sre_screen_write('RESET Using GMT Offset = 'sreparams.!gmtoffset)

oo=sreparams.!GMTOFFSET
foo=tsre_sysvars('GMTOFFSET',oo)

/* check existence of datadir */
datadir=sreparams.!DATADIR
datadir=translate(datadir,'\','/')
datadir=strip(strip(datadir,'t','\'))
if pos(':',datadir)=0 then do
   if abbrev(datadir,'\')=1 then do
        datadir=maindrive||datadir
   end
   else do
      datadir=maindir||'\'||datadir
   end /* do */
end
dd=dosisdir(datadir)
if dd=0 then do
   foo=sre_write_error( 'SRE2001. FATAL INI error: no such datadir ('datadir)
   return 0
end

sreparams.!datadir=datadir
foo=tsre_sysvars('DATADIR',datadir)

/* check limitbody and limitheader */
if datatype(sreparams.!limitbody)<>'NUM' then do
   foo=sre_write_error('SRE2001. FATAL INI error: LIMITBODY must be an integer > 0 ')
   return 0
end 
sreparams.!limitbody=sreparams.!limitbody*1000

/* check limitbody and limitheader */
if datatype(sreparams.!limitheader)<>'NUM' then do
   foo=sre_write_error( 'SRE2001. FATAL INI error: LIMITHEADER must be an integer > 0 ')
   exit
end 
sreparams.!limitheader=sreparams.!limitheader*1000

/* check cache_enable */
if sreparams.!req_cache_size=0 then sreparams.!req_cache_enable=0
if datatype(sreparams.!req_cache_record)<>'NUM' then sreparams.!req_cache_record=0


foo=macrodrop('SRE_FILTER')
foo=macroadd('SRE_FILTER',filter,'B')
if foo<>1 then do
    foo=sre_say( 'ERROR: unable to re-load main filter ('filter)
    return 0
end

/* reload fast-filter? */
if fast_filter<>0 then do
      foo=macrodrop('SRE_FAST_FILTER')
      foo=macroadd('SRE_FAST_FILTER',fast_filter,'B')
      if foo<>1 then do
         foo=sre_say( 'ERROR: unable to re-load fast filter ('fast_filter)
         return 0
      end
end

/* save "tsreparams" parameters "into macrospace" */
foo=save_tsreparams(tempdir)
if foo=0 then return 0

foo=sre_screen_write('RESET',' (parameters updated at '||time('n')||')')

return 1


/***********/
/* determine gmtoffset given tz variable */
fig_gmtoffset:procedure expose basicqueue  sreparams.  tsreparams.  sock1  sre2001_ini  pid
parse upper arg atz,defg
atz=strip(atz)

signal on syntax name foo2 
signal on error  name  foo2
signal on failure name foo2
signal on notready name foo2
curqueue=rxqueue('S',basicqueue)
'@bin\TZSET | RXQUEUE 2> nul '
do until queued()=0
   pull foo1
end 
foo=rxqueue('S',curqueue)

signal on syntax name abend
signal on error name abend
signal on failure name abend

foo1=space(foo1)
if abbrev(foo1,'THE CURRENT TIME ZONE')=0 then do
   foo=sre_audit('SRE2001.CMD',"ERROR Unexpected output from TZSET.EXE: "foo1)
   foo=sre_audit('SRE2001.CMD',"Using default gmtoffset= " defg,'111')
   return defg
end 
oo=word(foo1,words(foo1))
isneg=1
if left(oo,1)='-' then do 
  isneg=-1
  oo=substr(oo,2)
end 
if left(oo,1)='+' then do
   isneg=1
   oo=substr(oo,2)
end 
ii=length(oo)
minn=substr(oo,ii-1)
hrr=left(oo,ii-2)
aoff=-isneg*((hrr*3600)+(minn*60))
return aoff

foo2:
foo=sre_audit('SRE2001.CMD',"ERROR Unexpected output from TZSET.EXE: "foo1)
foo=sre_audit('SRE2001.CMD',"Using default gmtoffset= " defg,'111')
return defg

/*******/
showopts:
say
say "             "cy_ye"SRE2001 : An http/1.1 server for OS/2 "normal
say 
say cy_ye"Options: "normal
say bold"   no options:"normal" start the server, using options in "SRE2001_CFG
say bold"          ?  :"normal"  View this help screen "
say bold"  -i filename:"normal" Specify an initializaton file (instead of "SRE2001_CFG")"
say bold"  -p class_lvl:"normal" Specify priority class & level"
say bold"  -m [filename] :"normal" Modify most important initialization parameters -- "
say     "                  do NOT start the server"
say "                  (if filename not specified, modify "SRE2001_CFG") "
say bold"  -mall [filename] :"normal" Modify all initialization parameters -- "
say "                     do NOT start the server"
say "                     (if filename not specified, modify "SRE2001_CFG") "

say bold"         -cvt  :"normal"  Convert a file from GoServe-syntax to SRE2001-syntax "

say 
say "Examples (assuming the SRE2001 working directory is D:\SRE2001)"
say   bold"   D:\SRE2001>SRE2001 "normal
say   bold"   D:\SRE2001>SRE2001 -i SRE2001.in2 "normal
say   bold"   D:\SRE2001>SRE2001  -cvt  "normal
say   bold"   D:\SRE2001>SRE2001  -m  "normal
say   bold"   D:\SRE2001>SRE2001  -mall  "normal

say
return 0



/**********/
/* store "sysvars" for eventual writing 
Also, write them to SRE2001.ini         */

tsre_sysvars:procedure expose tsreparams.   sock1   sre2001_ini  pid
parse arg a1,a2
a1=strip(translate(a1))
aa='!'||a1
if wordpos(a1,tsreparams.0)=0 then tsreparams.0=tsreparams.0' 'a1
tsreparams.aa=a2
foo=sysini(sre2001_ini,'$SYSVARS',a1,a2)
return 0



/**********/
/* save tsreparams "into macrospace, by process id
   If port 80, also save by port  */
save_tsreparams:procedure expose tsreparams.  sock1  sre2001_ini  sreparams. pid
parse arg tempdir

fname=tempdir||'\SVR'||right(sreparams.!port,5,'$')||'.SRF'
foo=sysfiledelete(fname)

alist=tsreparams.0

call lineout fname,'/* sysvars macrospace procedure '||date('n')||' '||time('n')||' */'
call lineout fname,'parse arg avar,ownid,notaval'
call lineout fname,'PARAMS.="" '

call lineout fname,'avar=strip(translate(avar))'
do mm=1 to words(tsreparams.0) by 5
   xx=subword(tsreparams.0,mm,5)
    call lineout fname,"PARAMS.0=PARAMS.0||' "||xx||"'"
end 

call lineout fname,'if avar="!LIST" then return params.0'
do mm=1 to words(alist)
     aw='!'||strip(word(alist,mm))
     call lineout fname,'PARAMS.'||aw||"='"||tsreparams.aw"'"
end
call lineout fname,'aw="!"||strip(translate(avar))'
call lineout fname,'AVAL=params.aw'
call lineout fname,'if aval="" & notaval<>"" then do '
call lineout fname,' if wordpos(avar,params.0)=0 then aval=notaval'
call lineout fname,'end '
call lineout fname,'Return aval '
call lineout fname

AA=SRE_FLAG_set('#PAUSE_SYSVARS',1)         /* set the local PAUSE */
call syssleep 1                         /* wait a second */

nm1='SRE_'||pid||'_SYSVARS'
foo=macroadd(nm1,fname,'b')

if foo=0 then do
   call pmprintf("ERROR: unable to save system variables (in "fname") to macrospace "nm1)
   aA=SRE_FLAG_set('#PAUSE_SYSVARS',0)         /* also unset the local PAUSE */
   return 0
end
if sreparams.!port=80 then do
   nm1='SRE_P80_SYSVARS'
   foo=macroadd(nm1,fname,'b')
   aA=SRE_FLAG_set('#PAUSE_SYSVARS',0)         /* also unset the local PAUSE */
   if foo=0 then do
      call pmprintf("ERROR: unable to save system variables (in "fname") to macrospace "nm1)
      return 0
   end
end

return 1


/*****************/
/* load the various dlls and rxls  */
load_dlls: procedure expose  sre2001_ini  pid

/* Load various dlls */
foo=rxfuncquery('sysloadfuncs')
if foo=1 then do
  foo=RxFuncAdd( 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs')
  if foo=1 then do
     say "ERROR: could not find RexxUtil.DLL " 
     exit
  end /* do */
  call SysLoadFuncs
   
end

parse upper version rexxtype .
os2ver=sysos2ver()
say "  Using REXX version: " rexxtype ', under OS/2 ver 'os2ver


foo=rxfuncquery('rexxlibregister')
if foo=1 then do
 foo=rxfuncadd( 'rexxlibregister','rexxlib', 'rexxlibregister')
  if foo=1 then do
     say "ERROR: could not find RexxLib.DLL " 
     exit
  end /* do */
 call rexxlibregister
end

foo=RxFuncQuery("SockLoadFuncs") 
if foo=1 then do
    foo=RxFuncAdd( "SockLoadFuncs","rxSock","SockLoadFuncs")
    if foo=1 then do
       say "ERROR: could not find rxSock.DLL " 
       exit
    end /* do */
    call SockLoadFuncs
end
ver=sockversion()
if ver<2.0 then do
   say 'WARNING: old version of rxsocket ('ver'). We recommend using 2.0 or above'
end 


if rxfuncquery('srx_md5')=1  then do
  foo=RXFuncAdd( 'SRXLoadFuncs', 'SRXFUNC', 'SRXLoadFuncs')
   if foo=1 then do
       say "ERROR: could not find srxFunc.DLL " 
       exit
   end /* do */
  call SRXLoadFuncs
end
if rxfuncquery('srx_md5')=1  then do
       say "ERROR: could not find srx_Md5 in srxfunc.DLL " 
       exit
end

/* Load the rxgzlib functions */
if rxfuncquery('rxgzLoadFuncs')=1 then do
   foo=RxFuncAdd( 'rxgzLoadFuncs', 'rxgzLIB', 'rxgzLoadFuncs')
   if foo=1 then do
       say "ERROR: could not find rxGZlib.DLL " 
       exit
   end 
   call rxgzLoadFuncs
end 

if rxfuncquery('RxPrcLoadFuncs')=1 then do
 foo=RxFuncAdd('RxPrcLoadFuncs', 'RXPROC', 'RxPrcLoadFuncs')
 if foo=1 then do
    say "ERROR: could not find rxProc.DLL "
   exit
 end 
 call RxPrcLoadFuncs
end


/* load basic .rxl file */

arf='bin\SRE2001.IDX'       /* find list of procedures */
foo=stream(arf,'c','query size')
if foo=0 | foo=' ' then do
   say "ERROR: unable to clear procedures from macrospace"
   exit
end 
else do                 /* zap old versions of procedures */
   afoo=stream(arf,'c','open read')
   aa=charin(arf,1,foo)
   afoo=stream(arf,'c','close')

   idid=0
   do until aa=''
      parse var aa aline '0d0a'x aa
      if aline='' then iterate
      aline=strip(aline)
      if abbrev(aline,';')=1 then iterate
      if words(aline)>1 then iterate
      foo=macrodrop(aline)
      if foo<>1 then do
            if macroquery(aline)<>' ' then do
              say "ERROR: unable to macro_drop " aline
              exit
            end         /* otherwise, it's not there */
      end 
      else do
          idid=idid+1
      end
   end 
   say '  'idid' old versions of macrospace procedures removed.'
end 


/* load macrospace library */

rxlfile='bin\SRE2001.RXL'
if stream(rxlfile,'c','query exists')='' then do
      say "ERROR: "|| rxlfile ||" not available"
      exit
end 
aa=macroload(rxlfile)
if aa=0 then do
      say "ERROR: "|| rxlfile ||" was not loaded ("|| aa
      exit
end


return 1



